/*
****************************************
  Owner button functions for HMG 3.4.3

  Author:  KDJ
  Version: 2017-01-23
****************************************

Changelog:

2017-01-23
  + OBTN_CaptionAlign()
  * OBTN_Create()
    * third parameter now can be array {cCaption, cAlignment}
  * aColorEnabledButton, aColorDisabledButton, aColorFocusedButton: {nTextColor, nFrameColor, nBackColor1, nBackColor2, nGradientDirection[, cBitmapResource]}
    + cBitmapResource - bitmap that will be used to paint background instead of nBackColor1, nBackColor2, nGradientDirection

2016-12-09
  + gradiend for backgroud color

2016-12-04
  first release

****************************************

OBTN_Create(cParentFormName, nButtonID, [cCaption|aCaption], [nRow], [nCol], [nWidth], [nHeight], [lEnabled], [lVisible], [lTabStop], [nShape], [aColor], [aFont])
  --> nHWndButton
    aCaption: {cCaption, cAlignment}
    cAlignment: see OBTN_CaptionAlign()

OBTN_Release(cParentFormName, nButtonID)
  --> NIL

OBTN_Handle(cParentFormName, nButtonID)
  --> nHWndButton (Get)

GetDlgCtrlID(nHWndButton) - function is defined in HMG
  --> nButtonID (Get)

OBTN_Pos(cParentFormName, nButtonID, [nRow], [nCol], [nWidth], [nHeight])
  --> aPos {nRow, nCol, nWidth, nHeight} (Get/Set)

OBTN_Visible(cParentFormName, nButtonID, [lVisible])
  --> lVisible (Get/Set)

OBTN_Enable(cParentFormName, nButtonID, [lEnable])
  --> lEnabled (Get/Set)

OBTN_Focus(cParentFormName, nButtonID, [.T.])
  --> lFocused (Get/Set)

OBTN_Caption(cParentFormName, nButtonID, [cCaption])
  --> aCaption (Get/Set)

OBTN_CaptionAlign(cParentFormName, nButtonID, [cAlignment], [lRedraw])
  --> cAlignment (Get/Set)
    cAlignment contains two characters
      First char indicates the horizontal alignment, can be:
        "L" - left
        "R" - right
        "C" - center
      Second char indicates the vertical alignment, can be:
        "T" - top
        "B" - bottom
        "C" - center

OBTN_Shape(cParentFormName, nButtonID, [nShape], [lRedraw])
  --> nShape (Get/Set)

OBTN_Color(cParentFormName, nButtonID, [aColor], [lRedraw])
  --> aColor (Get/Set)

OBTN_Font(cParentFormName, nButtonID, [aFont], [lRedraw])
  --> aFont (Get/Set)

OBTN_Draw(nHWndParentForm, nButtonID, nDRAWITEMSTRUCT)
  --> NIL (to use only in event handler - if WM_DRAWITEM message is processed)

*****

nShape can be:
  -1  for ellipse
  >=0 for rectangle
  if >0 rectangle has rounded corners by a circle with diameter of nShape

aColor structure: {aColorEnabledButton, aColorDisabledButton, aColorFocusedButton}
  aColorEnabledButton, aColorDisabledButton, aColorFocusedButton: {nTextColor, nFrameColor, nBackColor1, nBackColor2, nGradientDirection[, cBitmapResource]}
    nTextColor, nFrameColor, nBackColor1, nBackColor2: 0x00bbggrr, eg:
      0x0000FF - red
      0x00FF00 - green
      0xFF0000 - blue
      0x000000 - black
      0x808080 - gray
      0xFFFFFF - white
    nGradientDirection - direction of the change background color from nBackColor1 to nBackColor2:
      0 - from top to bottom
      1 - from left to right
      2 - from outside to inside
    cBitmapResource - name of bitmap from .rc file that will be used to paint background instead of nBackColor1, nBackColor2, nGradientDirection
    If nFrameColor in aColorFocusedButton is greater than 0xFFFFFF, additional frame is not drawn.
    nBackColor1 and nBackColor2 are used to draw the background in gradient colors.

aFont structure: {cFontName, nFontSize, lBold, lItalic, lUnderline, lStrikeOut}
*/

#include "hmg.ch"

FUNCTION OBTN_Create(cForm, nID, aCaption, nRow, nCol, nWidth, nHeight, lEnabled, lVisible, lTabStop, nShape, aColor, aFont)
  LOCAL nTxColorE  := 0x000000
  LOCAL nFrColorE  := 0x000000
  LOCAL nBkColorE1 := 0xFFFFFF
  LOCAL nBkColorE2 := 0xFFFFFF
  LOCAL nGradDirE  := 0
  LOCAL cBmpResE   := ""
  LOCAL nTxColorD  := 0x808080
  LOCAL nFrColorD  := 0x808080
  LOCAL nBkColorD1 := 0XE0E0E0
  LOCAL nBkColorD2 := 0XE0E0E0
  LOCAL nGradDirD  := 0
  LOCAL cBmpResD   := ""
  LOCAL nTxColorF  := 0x000000
  LOCAL nFrColorF  := 0x000000
  LOCAL nBkColorF1 := 0xFFFFFF
  LOCAL nBkColorF2 := 0xFFFFFF
  LOCAL nGradDirF  := 0
  LOCAL cBmpResF   := ""
  LOCAL cFontName  := _HMG_SYSDATA[342]
  LOCAL nFontSize  := _HMG_SYSDATA[343]
  LOCAL lBold      := .F.
  LOCAL lItalic    := .F.
  LOCAL lUnderline := .F.
  LOCAL lStrikeOut := .F.
  LOCAL cCaption
  LOCAL cCAlign
  LOCAL nHButton

  IF HB_IsArray(aCaption)
    IF Len(aCaption) > 0
      cCaption := aCaption[1]
      IF Len(aCaption) > 1
        cCAlign := aCaption[2]
      ENDIF
    ENDIF
  ELSE
    cCaption := aCaption
  ENDIF

  nHButton := OwnButtonCreate(GetFormHandle(cForm), nID, HB_ValToStr(cCaption), nRow, nCol, nWidth, nHeight, lEnabled, lVisible, lTabStop)

  IF nHButton != 0
    IF HB_IsArray(aColor)
      IF HB_IsArray(aColor[1])
        IF HB_IsNumeric(aColor[1][1])
          nTxColorE := aColor[1][1]
        ENDIF
        IF HB_IsNumeric(aColor[1][2])
          nFrColorE := aColor[1][2]
        ENDIF
        IF HB_IsNumeric(aColor[1][3])
          nBkColorE1 := aColor[1][3]
        ENDIF
        IF HB_IsNumeric(aColor[1][4])
          nBkColorE2 := aColor[1][4]
        ENDIF
        IF HB_IsNumeric(aColor[1][5])
          nGradDirE := aColor[1][5]
        ENDIF
        IF (Len(aColor[1]) > 5) .and. HB_IsString(aColor[1][6])
          cBmpResE := aColor[1][6]
        ENDIF
      ENDIF

      IF HB_IsArray(aColor[2])
        IF HB_IsNumeric(aColor[2][1])
          nTxColorD := aColor[2][1]
        ENDIF
        IF HB_IsNumeric(aColor[2][2])
          nFrColorD := aColor[2][2]
        ENDIF
        IF HB_IsNumeric(aColor[2][3])
          nBkColorD1 := aColor[2][3]
        ENDIF
        IF HB_IsNumeric(aColor[2][4])
          nBkColorD2 := aColor[2][4]
        ENDIF
        IF HB_IsNumeric(aColor[2][5])
          nGradDirD := aColor[2][5]
        ENDIF
        IF (Len(aColor[2]) > 5) .and. HB_IsString(aColor[2][6])
          cBmpResD := aColor[2][6]
        ENDIF
      ENDIF

      IF HB_IsArray(aColor[3])
        IF HB_IsNumeric(aColor[3][1])
          nTxColorF := aColor[3][1]
        ENDIF
        IF HB_IsNumeric(aColor[3][2])
          nFrColorF := aColor[3][2]
        ENDIF
        IF HB_IsNumeric(aColor[3][3])
          nBkColorF1 := aColor[3][3]
        ENDIF
        IF HB_IsNumeric(aColor[3][4])
          nBkColorF2 := aColor[3][4]
        ENDIF
        IF HB_IsNumeric(aColor[3][5])
          nGradDirF := aColor[3][5]
        ENDIF
        IF (Len(aColor[3]) > 5) .and. HB_IsString(aColor[3][6])
          cBmpResF := aColor[3][6]
        ENDIF
      ENDIF
    ENDIF

    IF HB_IsArray(aFont)
      IF HB_IsString(aFont[1])
        cFontName := aFont[1]
      ENDIF
      IF HB_IsNumeric(aFont[2])
        nFontSize := aFont[2]
      ENDIF
      IF HB_IsLogical(aFont[3])
        lBold := aFont[3]
      ENDIF
      IF HB_IsLogical(aFont[4])
        lItalic := aFont[4]
      ENDIF
      IF HB_IsLogical(aFont[5])
        lUnderline := aFont[5]
      ENDIF
      IF HB_IsLogical(aFont[6])
        lStrikeOut := aFont[6]
      ENDIF
    ENDIF

    OBTN_CaptionAlign(cForm, nID, cCAlign)
    OBTN_Shape(cForm, nID, nShape)
    OBTN_Color(cForm, nID, {{nTxColorE, nFrColorE, nBkColorE1, nBkColorE2, nGradDirE, cBmpResE}, {nTxColorD, nFrColorD, nBkColorD1, nBkColorD2, nGradDirD, cBmpResD}, {nTxColorF, nFrColorF, nBkColorF1, nBkColorF2, nGradDirF, cBmpResF}})
    OBTN_Font(cForm, nID, {cFontName, nFontSize, lBold, lItalic, lUnderline, lStrikeOut})
  ENDIF

RETURN nHButton


FUNCTION OBTN_Release(cForm, nID)
  LOCAL nHParent := GetFormHandle(cForm)
  LOCAL nHButton := GetDlgItem(nHParent, nID)

  IF GetFocus() == nHButton
    SetFocus(GetNextDlgTabItem(nHParent, nHButton, .F.))
  ENDIF

  DestroyWindow(GetDlgItem(nHParent, nID))

RETURN NIL


FUNCTION OBTN_Handle(cForm, nID)

RETURN GetDlgItem(GetFormHandle(cForm), nID)


FUNCTION OBTN_Pos(cForm, nID, nRow, nCol, nWidth, nHeight)
  LOCAL nHParent := GetFormHandle(cForm)
  LOCAL nHButton := GetDlgItem(nHParent, nID)
  LOCAL aPos  := {0, 0, 0, 0}

  GetWindowRect(nHButton, aPos)
  aPos[3] := aPos[3] - aPos[1]
  aPos[4] := aPos[4] - aPos[2]
  ScreenToClient(nHParent, @aPos[1], @aPos[2])

  IF PCount() < 3
    RETURN {aPos[2], aPos[1], aPos[3], aPos[4]}
  ENDIF

  IF ! HB_IsNumeric(nRow)
    nRow := aPos[2]
  ENDIF

  IF ! HB_IsNumeric(nCol)
    nCol := aPos[1]
  ENDIF

  IF ! HB_IsNumeric(nWidth)
    nWidth := aPos[3]
  ENDIF

  IF ! HB_IsNumeric(nHeight)
    nHeight := aPos[4]
  ENDIF

  MoveWindow(nHButton, nCol, nRow, nWidth, nHeight, .T.)

RETURN OBTN_Pos(cForm, nID)


FUNCTION OBTN_Visible(cForm, nID, lVisible)
  LOCAL nHButton := GetDlgItem(GetFormHandle(cForm), nID)

  IF HB_IsLogical(lVisible)
    IF lVisible
      ShowWindow(nHButton)
    ELSE
      IF GetFocus() == nHButton
        SetFocus(GetNextDlgTabItem(GetFormHandle(cForm), nHButton, .F.))
      ENDIF

      HideWindow(nHButton)
    ENDIF
  ENDIF

RETURN IsWindowVisible(nHButton)


FUNCTION OBTN_Enable(cForm, nID, lEnable)
  LOCAL nHButton := GetDlgItem(GetFormHandle(cForm), nID)

  IF HB_IsLogical(lEnable)
    IF lEnable
      EnableWindow(nHButton)
    ELSE
      IF GetFocus() == nHButton
        SetFocus(GetNextDlgTabItem(GetFormHandle(cForm), nHButton, .F.))
      ENDIF

      DisableWindow(nHButton)
    ENDIF
  ENDIF

RETURN IsWindowEnabled(nHButton)


FUNCTION OBTN_Focus(cForm, nID, lFocus)
  LOCAL nHButton := GetDlgItem(GetFormHandle(cForm), nID)

  IF HB_IsLogical(lFocus) .and. lFocus .and. IsWindowEnabled(nHButton)
    SetFocus(nHButton)
  ENDIF

RETURN (GetFocus() == nHButton)


FUNCTION OBTN_Caption(cForm, nID, cCaption)
  LOCAL nHButton := GetDlgItem(GetFormHandle(cForm), nID)

  IF HB_IsString(cCaption)
    SetWindowText(nHButton, cCaption)
  ENDIF

RETURN GetWindowText(nHButton)


FUNCTION OBTN_CaptionAlign(cForm, nID, cAlignment, lRedraw)
  STATIC hCAlign := {=>}

  IF HB_IsString(cAlignment)
    cAlignment := HMG_PadR(HB_ULeft(HMG_Upper(cAlignment), 2), 2, "C")

    IF ! (HB_ULeft(cAlignment, 1) $ "LRC")
      cAlignment := HB_UTF8Stuff(cAlignment, 1, 1, "C")
    ENDIF

    IF ! (HB_URight(cAlignment, 1) $ "TBC")
      cAlignment := HB_UTF8Stuff(cAlignment, 2, 1, "C")
    ENDIF

    IF HB_hHaskey(hCAlign, cForm)
      hCAlign[cForm][nID] := cAlignment
    ELSE
      hCAlign[cForm] := {nID => cAlignment}
    ENDIF

    IF HB_IsLogical(lRedraw) .and. lRedraw
      RedrawWindow(GetDlgItem(GetFormHandle(cForm), nID))
    ENDIF
  ELSE
    IF HB_hHaskey(hCAlign, cForm) .and. HB_hHaskey(hCAlign[cForm], nID)
      cAlignment := hCAlign[cForm][nID]
    ELSE
      cAlignment := "CC"
    ENDIF
  ENDIF

RETURN cAlignment


FUNCTION OBTN_Font(cForm, nID, aFont, lRedraw)
  STATIC hFont := {=>}

  IF HB_IsArray(aFont)
    IF HB_hHaskey(hFont, cForm)
      IF HB_hHaskey(hFont[cForm], nID)
        IF HB_IsString(aFont[1])
          hFont[cForm][nID][1] := aFont[1]
        ENDIF
        IF HB_IsNumeric(aFont[2])
          hFont[cForm][nID][2] := aFont[2]
        ENDIF
        IF HB_IsLogical(aFont[3])
          hFont[cForm][nID][3] := aFont[3]
        ENDIF
        IF HB_IsLogical(aFont[4])
          hFont[cForm][nID][4] := aFont[4]
        ENDIF
        IF HB_IsLogical(aFont[5])
          hFont[cForm][nID][5] := aFont[5]
        ENDIF
        IF HB_IsLogical(aFont[6])
          hFont[cForm][nID][6] := aFont[6]
        ENDIF
      ELSE
        hFont[cForm][nID] := aFont
      ENDIF
    ELSE
      hFont[cForm] := {nID => aFont}
    ENDIF

    IF HB_IsLogical(lRedraw) .and. lRedraw
      RedrawWindow(GetDlgItem(GetFormHandle(cForm), nID))
    ENDIF
  ENDIF

RETURN If(HB_hHaskey(hFont, cForm) .and. HB_hHaskey(hFont[cForm], nID), hFont[cForm][nID], NIL)


FUNCTION OBTN_Shape(cForm, nID, nShape, lRedraw)
  STATIC hShape := {=>}

  IF HB_IsNumeric(nShape)
    IF HB_hHaskey(hShape, cForm)
      hShape[cForm][nID] := nShape
    ELSE
      hShape[cForm] := {nID => nShape}
    ENDIF

    IF HB_IsLogical(lRedraw) .and. lRedraw
      RedrawWindow(GetDlgItem(GetFormHandle(cForm), nID))
    ENDIF
  ELSE
    IF HB_hHaskey(hShape, cForm) .and. HB_hHaskey(hShape[cForm], nID)
      nShape := hShape[cForm][nID]
    ELSE
      nShape := 0
    ENDIF
  ENDIF

RETURN nShape


FUNCTION OBTN_Color(cForm, nID, aColor, lRedraw)
  STATIC hColor := {=>}

  IF HB_IsArray(aColor)
    IF HB_hHaskey(hColor, cForm)
      IF HB_hHaskey(hColor[cForm], nID)
        IF HB_IsArray(aColor[1])
          IF HB_IsNumeric(aColor[1][1])
            hColor[cForm][nID][1][1] := aColor[1][1]
          ENDIF
          IF HB_IsNumeric(aColor[1][2])
            hColor[cForm][nID][1][2] := aColor[1][2]
          ENDIF
          IF HB_IsNumeric(aColor[1][3])
            hColor[cForm][nID][1][3] := aColor[1][3]
          ENDIF
          IF HB_IsNumeric(aColor[1][4])
            hColor[cForm][nID][1][4] := aColor[1][4]
          ENDIF
          IF HB_IsNumeric(aColor[1][5])
            hColor[cForm][nID][1][5] := aColor[1][5]
          ENDIF
          IF (Len(aColor[1]) > 5) .and. HB_IsString(aColor[1][6])
            hColor[cForm][nID][1][6] := aColor[1][6]
          ENDIF
        ENDIF

        IF HB_IsArray(aColor[2])
          IF HB_IsNumeric(aColor[2][1])
            hColor[cForm][nID][2][1] := aColor[2][1]
          ENDIF
          IF HB_IsNumeric(aColor[2][2])
            hColor[cForm][nID][2][2] := aColor[2][2]
          ENDIF
          IF HB_IsNumeric(aColor[2][3])
            hColor[cForm][nID][2][3] := aColor[2][3]
          ENDIF
          IF HB_IsNumeric(aColor[2][4])
            hColor[cForm][nID][2][4] := aColor[2][4]
          ENDIF
          IF HB_IsNumeric(aColor[2][5])
            hColor[cForm][nID][2][5] := aColor[2][5]
          ENDIF
          IF (Len(aColor[2]) > 5) .and. HB_IsString(aColor[2][6])
            hColor[cForm][nID][2][6] := aColor[2][6]
          ENDIF
        ENDIF

        IF HB_IsArray(aColor[3])
          IF HB_IsNumeric(aColor[3][1])
            hColor[cForm][nID][3][1] := aColor[3][1]
          ENDIF
          IF HB_IsNumeric(aColor[3][2])
            hColor[cForm][nID][3][2] := aColor[3][2]
          ENDIF
          IF HB_IsNumeric(aColor[3][3])
            hColor[cForm][nID][3][3] := aColor[3][3]
          ENDIF
          IF HB_IsNumeric(aColor[3][4])
            hColor[cForm][nID][3][4] := aColor[3][4]
          ENDIF
          IF HB_IsNumeric(aColor[3][5])
            hColor[cForm][nID][3][5] := aColor[3][5]
          ENDIF
          IF (Len(aColor[3]) > 5) .and. HB_IsString(aColor[3][6])
            hColor[cForm][nID][3][6] := aColor[3][6]
          ENDIF
        ENDIF
      ELSE
        hColor[cForm][nID] := aColor
      ENDIF
    ELSE
      hColor[cForm] := {nID => aColor}
    ENDIF

    IF HB_IsLogical(lRedraw) .and. lRedraw
      RedrawWindow(GetDlgItem(GetFormHandle(cForm), nID))
    ENDIF
  ENDIF

RETURN If(HB_hHaskey(hColor, cForm) .and. HB_hHaskey(hColor[cForm], nID), hColor[cForm][nID], NIL)


FUNCTION OBTN_Draw(nHParent, nID, nDRAWITEMSTRUCT)
  LOCAL cForm   := GetFormNameByIndex(GetFormIndexByHandle(nHParent))
  LOCAL cCAlign := OBTN_CaptionAlign(cForm, nID)
  LOCAL nCAlign := 0x00
  LOCAL nShape  := OBTN_Shape(cForm, nID)
  LOCAL aColor  := OBTN_Color(cForm, nID)
  LOCAL aFont   := OBTN_Font(cForm, nID)

  IF HB_ULeft(cCAlign, 1) == "L"
    nCAlign := HB_BitOr(nCAlign, 0x01)
  ELSEIF HB_ULeft(cCAlign, 1) == "R"
    nCAlign := HB_BitOr(nCAlign, 0x02)
  ENDIF

  IF HB_URight(cCAlign, 1) == "T"
    nCAlign := HB_BitOr(nCAlign, 0x10)
  ELSEIF HB_URight(cCAlign, 1) == "B"
    nCAlign := HB_BitOr(nCAlign, 0x20)
  ENDIF

  OwnButtonDraw(nDRAWITEMSTRUCT, nShape, aColor[1], aColor[2], aColor[3], aFont, nCAlign)

RETURN NIL


#pragma BEGINDUMP

#include "SET_COMPILE_HMG_UNICODE.ch"
#include "HMG_UNICODE.h"
#include <windows.h>
#include <commctrl.h>
#include <math.h>
#include "hbapi.h"

static HBRUSH OwnButtonBrushCreate(HDC hDC, INT nWidth, INT nHeight, COLORREF Color1, COLORREF Color2, INT nDirection, LPCTSTR BmpRes)
{
  INT     R1, G1, B1, R2, G2, B2;
  HDC     hDCComp;
  HBITMAP hBitmap;
  HBRUSH  hBrush, hBrushPat;
  RECT    rcF;
  INT     nCount;
  INT     i;

  R1 = GetRValue(Color1);
  G1 = GetGValue(Color1);
  B1 = GetBValue(Color1);
  R2 = GetRValue(Color2);
  G2 = GetGValue(Color2);
  B2 = GetBValue(Color2);

  hDCComp = CreateCompatibleDC(hDC);
  hBitmap = LoadImage(GetModuleHandle(NULL), BmpRes, IMAGE_BITMAP, nWidth, nHeight, LR_DEFAULTCOLOR);

  if (hBitmap)
  {
    SelectObject(hDCComp, hBitmap);
  }
  else
  {
    hBitmap = CreateCompatibleBitmap(hDC, nWidth, nHeight);

    SelectObject(hDCComp, hBitmap);

    rcF.right  = nWidth;
    rcF.bottom = nHeight;

    if (nDirection == 0)
    {
      rcF.left = 0;

      for (i = 0; i < nHeight; i++)
      { 
        rcF.top = i;
        hBrush  = CreateSolidBrush(RGB(R1 + (i * (R2 - R1) / nHeight), G1 + (i * (G2 - G1) / nHeight), B1 + (i * (B2 - B1) / nHeight)));
        FillRect(hDCComp, &rcF, hBrush);
        DeleteObject(hBrush);
      }
    }
    else if (nDirection == 1)
    {
      rcF.top = 0;

      for (i = 0; i < nWidth; i++)
      { 
        rcF.left = i;
        hBrush   = CreateSolidBrush(RGB(R1 + (i * (R2 - R1) / nWidth), G1 + (i * (G2 - G1) / nWidth), B1 + (i * (B2 - B1) / nWidth)));
        FillRect(hDCComp, &rcF, hBrush);
        DeleteObject(hBrush);
      }
    }
    else
    {
      rcF.left = 0;
      rcF.top  = 0;
      nCount   = ceil(((nWidth > nHeight) ? nHeight : nWidth) / 2);

      for (i = 0; i < nCount; i++)
      { 
        hBrush = CreateSolidBrush(RGB(R1 + (i * (R2 - R1) / nCount), G1 + (i * (G2 - G1) / nCount), B1 + (i * (B2 - B1) / nCount)));
        FillRect(hDCComp, &rcF, hBrush);
        DeleteObject(hBrush);

        ++rcF.left;
        ++rcF.top;
        --rcF.right;
        --rcF.bottom;
      }
    }
  }

  hBrushPat = CreatePatternBrush(hBitmap);

  DeleteDC(hDCComp);
  DeleteObject(hBitmap);

  return hBrushPat;
}


       //                1         2    3          4       5       6         7          8           9           10
       //OwnButtonCreate(nHParent, nID, cCaption, [nRow], [nCol], [nWidth], [nHeight], [lEnabled], [lVisible], [lTabStop])
HB_FUNC( OWNBUTTONCREATE )
{
  DWORD  dwStyle = BS_OWNERDRAW | WS_CHILD;
  HWND hButton;

  if (! hb_parldef(8, 1))
    dwStyle |= WS_DISABLED;
  if (hb_parldef(9, 1))
    dwStyle |= WS_VISIBLE;
  if (hb_parldef(10, 1))
    dwStyle |= WS_TABSTOP;

  hButton = CreateWindowEx(0,
                           (LPCTSTR) WC_BUTTON,
                           (LPCTSTR) HMG_parc(3),
                           dwStyle,
                           hb_parnidef(5, 0),
                           hb_parnidef(4, 0),
                           hb_parnidef(6, 0),
                           hb_parnidef(7, 0),
                           (HWND) HMG_parnl(1),
                           (HMENU) HMG_parnl(2),
                           GetModuleHandle(NULL),
                           NULL);
  HMG_retnl((LONG_PTR) hButton);
}


       //              1                2       3        4        5        6      7
       //OwnButtonDraw(nDRAWITEMSTRUCT, nShape, aColor1, aColor2, aColor3, aFont, nCAlign)
HB_FUNC( OWNBUTTONDRAW )
{
  DRAWITEMSTRUCT *pDIS = (DRAWITEMSTRUCT *) HMG_parnl(1);

  INT   nShape     = (INT) hb_parni(2);
  TCHAR *FontName  = (TCHAR *) HMG_parvc(6, 1);
  INT   nFontSize  = (INT)     hb_parvni(6, 2);
  INT   nBold      = hb_parvl(6, 3) ? FW_BOLD : FW_NORMAL;
  INT   nItalic    = hb_parvl(6, 4) ? 1 : 0;
  INT   nUnderline = hb_parvl(6, 5) ? 1 : 0;
  INT   nStrikeOut = hb_parvl(6, 6) ? 1 : 0;
  INT   nCAlign    = (INT) hb_parni(7);

  COLORREF TextColor;
  COLORREF FrameColor;
  COLORREF FrameColor2 = 0xFFFFFFFF;
  COLORREF BackColor1;
  COLORREF BackColor2;
  INT      nGradDir;
  LPCTSTR  BmpRes;
  HBRUSH   hBrush;
  HBRUSH   hBrush2;
  HBRUSH   hBrushOld;
  HPEN     hPen;
  HPEN     hPenOld;
  HFONT    hFont;
  HFONT    hFontOld;
  RECT     rcText;
  INT      nTextH;
  UINT     uFormat;

  INT nTextLen = GetWindowTextLength(pDIS->hwndItem) + 1;
  TCHAR *Text  = (TCHAR *) hb_xgrab(nTextLen * sizeof(TCHAR));
  GetWindowText(pDIS->hwndItem, Text, nTextLen);

  if (pDIS->itemState & ODS_DISABLED)
  {
    TextColor  = (COLORREF) hb_parvni(4, 1);
    FrameColor = (COLORREF) hb_parvni(4, 2);
    BackColor1 = (COLORREF) hb_parvni(4, 3);
    BackColor2 = (COLORREF) hb_parvni(4, 4);
    nGradDir   = (INT)      hb_parvni(4, 5);
    BmpRes     = (LPCTSTR)  HMG_parvc(4, 6);
  }
  else if (pDIS->itemState & ODS_FOCUS)
  {
    TextColor   = (COLORREF) hb_parvni(5, 1);
    FrameColor  = (COLORREF) hb_parvni(3, 2);
    FrameColor2 = (COLORREF) hb_parvni(5, 2);
    BackColor1  = (COLORREF) hb_parvni(5, 3);
    BackColor2  = (COLORREF) hb_parvni(5, 4);
    nGradDir    = (INT)      hb_parvni(5, 5);
    BmpRes      = (LPCTSTR)  HMG_parvc(5, 6);
  }
  else
  {
    TextColor  = (COLORREF) hb_parvni(3, 1);
    FrameColor = (COLORREF) hb_parvni(3, 2);
    BackColor1 = (COLORREF) hb_parvni(3, 3);
    BackColor2 = (COLORREF) hb_parvni(3, 4);
    nGradDir   = (INT)      hb_parvni(3, 5);
    BmpRes     = (LPCTSTR)  HMG_parvc(3, 6);
  }

  hBrush = OwnButtonBrushCreate(pDIS->hDC, (pDIS->rcItem.right - pDIS->rcItem.left), (pDIS->rcItem.bottom - pDIS->rcItem.top), BackColor1, BackColor2, nGradDir, BmpRes);
  hPen   = CreatePen(PS_INSIDEFRAME, 1, FrameColor);
  hFont  = CreateFont(-MulDiv(nFontSize, GetDeviceCaps(pDIS->hDC, LOGPIXELSY), 72), 0, 0, 0, nBold, nItalic, nUnderline, nStrikeOut, DEFAULT_CHARSET, OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, FontName);

  hBrushOld = SelectObject(pDIS->hDC, hBrush);
  hPenOld   = SelectObject(pDIS->hDC, hPen);
  hFontOld  = SelectObject(pDIS->hDC, hFont);

  if (FrameColor2 <= 0xFFFFFF)
  {
    hBrush2 = CreateSolidBrush(FrameColor2);
    SelectObject(pDIS->hDC, hBrush2);

    if (nShape >= 0)
      RoundRect(pDIS->hDC, pDIS->rcItem.left, pDIS->rcItem.top, pDIS->rcItem.right, pDIS->rcItem.bottom, nShape, nShape);
    else
      Ellipse(pDIS->hDC, pDIS->rcItem.left, pDIS->rcItem.top, pDIS->rcItem.right, pDIS->rcItem.bottom);

    SelectObject(pDIS->hDC, hBrush);
    DeleteObject(hBrush2);

    if (nShape >= 0)
      RoundRect(pDIS->hDC, pDIS->rcItem.left + 2, pDIS->rcItem.top + 2, pDIS->rcItem.right - 2, pDIS->rcItem.bottom - 2, nShape - 2, nShape - 2);
    else
      Ellipse(pDIS->hDC, pDIS->rcItem.left + 2, pDIS->rcItem.top + 2, pDIS->rcItem.right - 2, pDIS->rcItem.bottom - 2);
  }
  else
  {
    if (nShape >= 0)
      RoundRect(pDIS->hDC, pDIS->rcItem.left, pDIS->rcItem.top, pDIS->rcItem.right, pDIS->rcItem.bottom, nShape, nShape);
    else
      Ellipse(pDIS->hDC, pDIS->rcItem.left, pDIS->rcItem.top, pDIS->rcItem.right, pDIS->rcItem.bottom);
  }

  SetTextColor(pDIS->hDC, TextColor);
  SetBkMode(pDIS->hDC, TRANSPARENT);

  rcText.left   = pDIS->rcItem.left;
  rcText.top    = pDIS->rcItem.top ;
  rcText.right  = pDIS->rcItem.right;
  rcText.bottom = pDIS->rcItem.bottom;

  nTextH = DrawText(pDIS->hDC, Text, -1, &rcText, DT_CALCRECT | DT_NOCLIP);

  rcText.left  = pDIS->rcItem.left  + 4;
  rcText.right = pDIS->rcItem.right - 4;

  if (nTextH < (pDIS->rcItem.bottom - pDIS->rcItem.top - 2 * 4))
  {
    if (nCAlign & 0x10)
      rcText.top = pDIS->rcItem.top + 4;
    else if (nCAlign & 0x20)
      rcText.top = pDIS->rcItem.bottom - nTextH - 4;
    else
      rcText.top = (pDIS->rcItem.bottom - pDIS->rcItem.top - nTextH) / 2;

    rcText.bottom = rcText.top + nTextH;
  }
  else
  {
    rcText.top    = pDIS->rcItem.top    + 4;
    rcText.bottom = pDIS->rcItem.bottom - 4;
  }

  if (nCAlign & 0x01)
    uFormat = DT_LEFT;
  else if (nCAlign & 0x02)
    uFormat = DT_RIGHT;
  else
    uFormat = DT_CENTER;

  DrawText(pDIS->hDC, Text, -1, &rcText, uFormat);

  SelectObject(pDIS->hDC, hBrushOld);
  SelectObject(pDIS->hDC, hPenOld);
  SelectObject(pDIS->hDC, hFontOld);

  DeleteObject(hBrush);
  DeleteObject(hPen);
  DeleteObject(hFont);
  hb_xfree(Text);
}

#pragma ENDDUMP
